@@ -8,9 +8,13 @@ sub new
     return bless {}, $class;
 }
 
-sub maketext { return @_ };
+sub maketext
+{
+    my $self = shift;
+    return join '', @_;
+}
 
 1;
 
 
-__END__ 
+__END__
